@@ -160,6 +160,9 @@ swf_object_output(swf_object_t *swf, unsigned long *length) {
     bitstream_close(bs);
     return data;
 }
+
+/* --- */
+
 void
 swf_object_print(swf_object_t *swf) {
     int i;
@@ -169,7 +172,7 @@ swf_object_print(swf_object_t *swf) {
     tag = swf->tag;
     for (i=0 ; tag ; i++) {
         printf("[%d] ", i);
-        swf_tag_print(tag, swf);
+        swf_tag_print(tag, swf, 0);
         if (tag->tag == 0) { // END Tag
             break;
         }
@@ -177,6 +180,43 @@ swf_object_print(swf_object_t *swf) {
     }
 }
 
+extern unsigned char *
+swf_object_get_tagdata(swf_object_t *swf, int tag_seqno,
+                       unsigned long *length) {
+    int i;
+    swf_tag_t *tag;
+    tag = swf->tag;
+    for (i=0 ; (i < tag_seqno) &&  tag ; i++) {
+        tag = tag->next;
+    }
+    if (tag) {
+        if (tag->data) {
+            *length = tag->length;
+            return tag->data;
+        }
+        if (tag->detail == NULL) {
+            bitstream_t *bs;
+            unsigned char *data;
+            bs = bitstream_open();
+            swf_tag_build(bs, tag, swf);
+            data = bitstream_steal(bs, length);
+            bitstream_close(bs);
+            return data;
+        }
+        if (tag->detail) {
+            *length = tag->length;
+            return tag->data;
+        }
+    }
+    return NULL;
+}
+
+int
+swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
+                           unsigned char *data, unsigned long *length) {
+    fprintf(stderr, "swf_object_replace_tagdata: not impremented yet.\n");
+    return 1;
+}
 
 /* --- */
 
@@ -257,6 +297,7 @@ swf_object_replace_jpegdata(swf_object_t *swf, int image_id,
     return result;
 }
 
+#ifdef HAVE_PNG
 
 unsigned char *
 swf_object_get_pngdata(swf_object_t *swf, unsigned long *length, int image_id) {
@@ -300,6 +341,11 @@ swf_object_replace_pngdata(swf_object_t *swf, int image_id,
     return result;
 }
 
+
+#endif /* HAVE_PNG */
+
+#ifdef HAVE_GIF
+
 int
 swf_object_replace_gifdata(swf_object_t *swf, int image_id,
                             unsigned char *gif_data,
@@ -320,6 +366,8 @@ swf_object_replace_gifdata(swf_object_t *swf, int image_id,
     return result;
 }
 
+#endif /* HAVE_GIF */
+
 unsigned char *
 swf_object_get_sounddata(swf_object_t *swf, unsigned long *length, int sound_id) {
     swf_tag_t *tag;
