@@ -14,10 +14,13 @@
 #include "swf_tag_edit.h"
 #include "swf_tag_action.h"
 #include "swf_tag_sound.h"
+#include "swf_tag_sprite.h"
+#include "swf_tag_shape.h"
 
 swf_tag_info_t swf_tag_info_table[] = {
     { 0, "End", NULL },
     { 1, "ShowFrame", NULL },
+    //    { 2, "DefineShape", swf_tag_shape_detail_handler },
     { 2, "DefineShape", NULL },
     { 3, "FreeCharacter", NULL},
     { 4, "PlaceObject", NULL},
@@ -38,17 +41,21 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 19, "SoundStreamBlock", NULL },
     { 20, "DefineBitsLossless", swf_tag_lossless_detail_handler },
     { 21, "DefineBitsJPEG2", swf_tag_jpeg_detail_handler },
+    //    { 22, "DefineShape2", swf_tag_shape_detail_handler },
     { 22, "DefineShape2", NULL },
     { 26, "PlaceObject2", NULL },
     { 28, "RemoveObject2", NULL },
+    //    { 32, "DefineShape3", swf_tag_shape_detail_handler },
     { 32, "DefineShape3", NULL },
     { 33, "DefineText2", NULL },
     { 34, "DefineButton2", NULL },
     { 35, "DefineBitsJPEG3", swf_tag_jpeg3_detail_handler },
     { 36, "DefineBitsLossless2", swf_tag_lossless_detail_handler },
     { 37, "DefineEditText", swf_tag_edit_detail_handler },
-    { 39, "DefineSprite", NULL } ,
+    { 39, "DefineSprite", swf_tag_sprite_detail_handler },
     { 43, "FrameLabel", NULL } ,
+    //    { 46, "DefineMorphShape", swf_tag_shape_detail_handler },
+    { 46, "DefineMorphShape", NULL },
     { 48, "DefineFont2", NULL } ,
     { 56, "Export", NULL } ,
     { 59, "DoInitAction", NULL } ,
@@ -56,7 +63,10 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 73, "DefineFontAlignZones", NULL },
     { 74, "CSMTextSettings", NULL },
     { 75, "DefineFont3", NULL } ,
+    //    { 83, "DefineShape4", swf_tag_shape_detail_handler },
     { 83, "DefineShape4", NULL },
+    //    { 84, "DefineMorphShape2", swf_tag_shape_detail_handler },
+    { 84, "DefineMorphShape2", NULL},
     { 88, "DefineFontName", NULL } ,
     { 777,"Reflex", NULL } ,
 };
@@ -201,7 +211,7 @@ extern int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *sw
 }
 
 void
-swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf) {
+swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth) {
     swf_tag_info_t *tag_info;
     const char *tag_name;
     if (tag == NULL) {
@@ -210,6 +220,7 @@ swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf) {
     }
     tag_info = get_swf_tag_info(tag->tag);
     tag_name = (tag_info)?tag_info->name:"Unknown";
+//  print_indent(indent_depth);
     printf("tag=%s(%d)", tag_name, tag->tag);
     if (tag->length > 0) {
         printf("  length=%lu",  tag->length);
@@ -221,7 +232,7 @@ swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf) {
         }
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
         if (detail_handler->print) {
-            detail_handler->print(tag, swf);
+            detail_handler->print(tag, swf, indent_depth + 1);
         }
     }
 }
@@ -366,6 +377,9 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
     return result;
 }
 
+
+#ifdef HAVE_PNG
+
 unsigned char *
 swf_tag_get_png_data(swf_tag_t *tag, unsigned long *length, int image_id) {
     swf_tag_info_t *tag_info;
@@ -436,6 +450,10 @@ swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
     return result;
 }
 
+#endif /* HAVE_PNG */
+
+#ifdef HAVE_GIF
+
 int
 swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
                          unsigned char *gif_data,
@@ -484,6 +502,8 @@ swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
     return result;
 }
 
+#endif /* HAVE_GIF */
+
 /*
  * DefineSound
  */
