@@ -110,8 +110,10 @@ swf_tag_jpeg3_input_detail(swf_tag_t *tag,
     swf_tag_jpeg->jpeg_data_len = offset_to_alpha;
     offset = 2 + 4 + offset_to_alpha;
     alpha_data_len = length - offset;
-    origsize = 256 * alpha_data_len;
-    new_buff = malloc(origsize); // enough size
+    // 本来は jpeg を解読して width*height で算出するべきだが…
+    origsize = 512 * alpha_data_len; // XXX jpeg1,2より少し大目
+    
+    new_buff = malloc(origsize); // enough size?
     old_buff_ref = bitstream_buffer(bs, offset);
     result = uncompress(new_buff, &origsize, old_buff_ref, alpha_data_len);
     if (result == Z_OK) {
@@ -212,7 +214,7 @@ swf_tag_jpeg3_output_detail(swf_tag_t *tag, unsigned long *length,
 
 void
 swf_tag_jpeg_print_detail(swf_tag_t *tag,
-                          struct swf_object_ *swf) {
+                          struct swf_object_ *swf, int indent_depth) {
     swf_tag_jpeg_detail_t *swf_tag_jpeg = (swf_tag_jpeg_detail_t *) tag->detail;
     jpeg_segment_t *jpeg_seg;
     jpeg_segment_node_t *node;
@@ -221,22 +223,26 @@ swf_tag_jpeg_print_detail(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_jpeg_print_detail: swf_tag_jpeg == NULL\n");
         return ;
     }
-    printf("\timage_id=%d  jpeg_data_size=%lu\n",
+    print_indent(indent_depth);
+    printf("image_id=%d  jpeg_data_size=%lu\n",
            swf_tag_jpeg->image_id, swf_tag_jpeg->jpeg_data_len);
     jpeg_seg = jpeg_segment_parse(swf_tag_jpeg->jpeg_data,
                                   swf_tag_jpeg->jpeg_data_len);
     if (jpeg_seg) {
         for (node=jpeg_seg->head ; node ; node=node->next) {
             char *name = jpeg_segment_get_marker_name(node->marker);
-            printf("\t\t%s(0x%02X): len=%lu\n", name?name:"Unknwon",
+            print_indent(indent_depth + 1);
+            printf("%s(0x%02X): len=%lu\n", name?name:"Unknwon",
                    node->marker, node->data_len);
         }
         jpeg_segment_destroy(jpeg_seg);
     } else {
-        printf("\t\t(invalid jpeg data)\n");
+        print_indent(indent_depth + 1);
+        printf("(invalid jpeg data)\n");
     }
     if (swf_tag_jpeg->alpha_data) {
-        printf("  alpha_data_size=%lu\n",
+        print_indent(indent_depth);
+        printf("alpha_data_size=%lu\n",
                swf_tag_jpeg->alpha_data_len);
     }
 }
@@ -314,19 +320,21 @@ swf_tag_jpeg_replace_jpeg_data(void *detail, int image_id,
     }
     swf_tag_jpeg->image_id = image_id;
     if (tag->tag == 6) { // DefineBitsJPEG
-        free(swf_tag_jpeg->jpeg_data);
-        swf_tag_jpeg->jpeg_data = malloc(jpeg_data_len);
-        memcpy(swf_tag_jpeg->jpeg_data, jpeg_data, jpeg_data_len);
-        swf_tag_jpeg->jpeg_data_len = jpeg_data_len;
+        if (jpeg_data && jpeg_data_len) { // fail safe
+            free(swf_tag_jpeg->jpeg_data);
+            swf_tag_jpeg->jpeg_data = malloc(jpeg_data_len);
+            memcpy(swf_tag_jpeg->jpeg_data, jpeg_data, jpeg_data_len);
+            swf_tag_jpeg->jpeg_data_len = jpeg_data_len;
+        }
     } else {
-        if (jpeg_data) {
+        if (jpeg_data && jpeg_data_len) {
             unsigned long length;
             free(swf_tag_jpeg->jpeg_data);
             swf_tag_jpeg->jpeg_data = jpegconv_std2swf(jpeg_data, jpeg_data_len,
                                                        &length);
             swf_tag_jpeg->jpeg_data_len = length;
         }
-        if (alpha_data) {
+        if (alpha_data && alpha_data_len) {
             free(swf_tag_jpeg->alpha_data);
             swf_tag_jpeg->alpha_data = malloc(alpha_data_len);
             memcpy(swf_tag_jpeg->alpha_data, alpha_data, alpha_data_len);
