@@ -298,19 +298,21 @@ swf_tag_lossless_output_detail(swf_tag_t *tag, unsigned long *length,
 
 void
 swf_tag_lossless_print_detail(swf_tag_t *tag,
-                              struct swf_object_ *swf) {
+                              struct swf_object_ *swf, int indent_depth) {
     swf_tag_lossless_detail_t *swf_tag_lossless = (swf_tag_lossless_detail_t *) tag->detail;
     (void) swf;
     if (swf_tag_lossless == NULL) {
         fprintf(stderr, "swf_tag_lossless_print_detail: swf_tag_lossless == NULL\n");
         return ;
     }
-    printf("\timage_id=%d  format=%d  width=%u  height=%u\n",
+    print_indent(indent_depth);
+    printf("image_id=%d  format=%d  width=%u  height=%u\n",
            swf_tag_lossless->image_id, swf_tag_lossless->format,
            swf_tag_lossless->width, swf_tag_lossless->height);
     if (swf_tag_lossless->colormap ||
         swf_tag_lossless->colormap2) {
-        printf("\tcolormap_count=%d",
+        print_indent(indent_depth);
+        printf("colormap_count=%d",
                swf_tag_lossless->colormap_count);
         if (swf_tag_lossless->colormap) {
             printf("  rgb colormap exists");
@@ -323,10 +325,12 @@ swf_tag_lossless_print_detail(swf_tag_t *tag,
         printf("\n");
     }
     if (swf_tag_lossless->bitmap) {
-        printf("\txrgb bitmap exists\n");
+        print_indent(indent_depth);
+        printf("xrgb bitmap exists\n");
     }
     if (swf_tag_lossless->bitmap2) {
-        printf("\targb bitmap exists\n");
+        print_indent(indent_depth);
+        printf("argb bitmap exists\n");
     }
 }
 
@@ -348,7 +352,9 @@ swf_tag_lossless_destroy_detail(swf_tag_t *tag) {
     }
     return ;
 }
-    
+
+#ifdef HAVE_PNG
+
 unsigned char *swf_tag_lossless_get_png_data(void *detail,
                                           unsigned long *length,
                                              int image_id,
@@ -474,6 +480,10 @@ swf_tag_lossless_replace_png_data(void *detail, int image_id,
     return 0;
 }
 
+#endif /* HAVE_PNG */
+
+#ifdef HAVE_GIF
+
 int
 swf_tag_lossless_replace_gif_data(void *detail, int image_id,
                                   unsigned char *gif_data,
@@ -529,3 +539,5 @@ swf_tag_lossless_replace_gif_data(void *detail, int image_id,
     }
     return 0;
 }
+
+#endif /* HAVE_GIF */
