@@ -33,6 +33,7 @@
 #include "swf_tag_edit.h"
 #include "swf_tag_sound.h"
 #include "swf_tag_action.h"
+#include "swf_tag_sprite.h"
 #include "swf_tag.h"
 #include "swf_object.h"
 
@@ -56,6 +57,7 @@ zend_function_entry swfed_functions[] = {
       	PHP_ME(swfed,  getTagList, NULL, 0)
    	PHP_ME(swfed,  getTagDetail, NULL, 0)
        	PHP_ME(swfed,  getTagData, NULL, 0)
+        PHP_ME(swfed,  replaceTagData, NULL, 0)
    	PHP_ME(swfed,  getJpegData, NULL, 0)
    	PHP_ME(swfed,  getJpegAlpha, NULL, 0)
    	PHP_ME(swfed,  replaceJpegData, NULL, 0)
@@ -80,14 +82,14 @@ zend_module_entry swfed_module_entry = {
 	STANDARD_MODULE_HEADER,
 #endif
 	"swfed",
-	swfed_functions,
+	NULL, // no global functions
 	PHP_MINIT(swfed),
 	PHP_MSHUTDOWN(swfed),
 	PHP_RINIT(swfed),		/* Replace with NULL if there's nothing to do at request start */
 	PHP_RSHUTDOWN(swfed),	/* Replace with NULL if there's nothing to do at request end */
 	PHP_MINFO(swfed),
 #if ZEND_MODULE_API_NO >= 20010901
-	"0.1", /* Replace with version number for your extension */
+	"0.20", /* Replace with version number for your extension */
 #endif
 	STANDARD_MODULE_PROPERTIES
 };
@@ -176,8 +178,18 @@ PHP_RSHUTDOWN_FUNCTION(swfed)
  */
 PHP_MINFO_FUNCTION(swfed)
 {
+	int gif_support = 0;
+	int png_support = 0;
 	php_info_print_table_start();
-	php_info_print_table_header(2, "swfed support", "enabled");
+	php_info_print_table_header(2, "SWF Editor support", "enabled");
+#ifdef HAVE_PNG
+	png_support = 1;
+#endif
+#ifdef HAVE_GIF
+	gif_support = 1;
+#endif
+	php_info_print_table_row(2, "SWF Editor PNG support", png_support?"yes":"no");
+	php_info_print_table_row(2, "SWF Editor GIF support", gif_support?"yes":"no");
 	php_info_print_table_end();
 
 	/* Remove comments if you have entries in php.ini
@@ -378,6 +390,7 @@ PHP_METHOD(swfed, getTagDetail) {
         swf_tag_edit_detail_t     *tag_edit;
         swf_tag_sound_detail_t    *tag_sound;
         swf_tag_action_detail_t   *tag_action;
+        swf_tag_sprite_detail_t   *tag_sprite;
       case 6:  // DefineBitsJPEG
       case 21: // DefineBitsJPEG2
       case 35: // DefineBitsJPEG3
@@ -439,6 +452,12 @@ PHP_METHOD(swfed, getTagDetail) {
                                 (char *)tag_edit->edit_initial_text, 1);
         }
         break;
+      case 39: // DefineSprite;
+        tag_sprite = tag->detail;
+        array_init(return_value);
+        add_assoc_long(return_value, "sprite_id", tag_sprite->sprite_id);
+        add_assoc_long(return_value, "frame_count", tag_sprite->frame_count);
+        break;
       default:
         RETURN_FALSE;
     }
@@ -446,8 +465,54 @@ PHP_METHOD(swfed, getTagDetail) {
 }
 
 PHP_METHOD(swfed, getTagData) {
-    printf("not implemented yet.\n");
-    RETURN_FALSE;
+    long tag_seqno;
+    swf_object_t *swf;
+    unsigned char *data_ref;
+    char *new_buff;
+    unsigned long data_len;
+    
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &tag_seqno) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    data_ref = swf_object_get_tagdata(swf, tag_seqno, &data_len);
+    if (data_ref == NULL) {
+        fprintf(stderr, "getTagData: Can't get_tagdata\n");
+        RETURN_FALSE;
+    }
+    new_buff = emalloc(data_len);
+    if (new_buff == NULL) {
+        fprintf(stderr, "getTagData: Can't emalloc new_buff\n");
+        RETURN_FALSE;
+    }
+    memcpy(new_buff, data_ref, data_len);
+    RETURN_STRINGL(new_buff, data_len, 0);
+}
+
+PHP_METHOD(swfed, replaceTagData) {
+    char *data = NULL;
+    unsigned long data_len = 0;
+    int tag_seqno;
+    swf_object_t *swf;
+    int result = 0;
+    switch (ZEND_NUM_ARGS()) {
+      default:
+        WRONG_PARAM_COUNT;
+        RETURN_FALSE; /* XXX */
+      case 2:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &tag_seqno, &data, &data_len) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    result = swf_object_replace_tagdata(swf, tag_seqno,
+                                        (unsigned char *)data,
+                                        & data_len);
+    if (result) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
 }
 
 PHP_METHOD(swfed, getJpegData) {
@@ -536,6 +601,10 @@ PHP_METHOD(swfed, replaceJpegData) {
 }
 
 PHP_METHOD(swfed, getPNGData) {
+#ifndef HAVE_PNG
+    fprintf(stderr, "replacePNGData: no png library\n");
+    RETURN_FALSE;
+#else /* HAVE_PNG */
     unsigned long image_id = 0;
     unsigned long len = 0;
     unsigned char *data;
@@ -559,9 +628,14 @@ PHP_METHOD(swfed, getPNGData) {
     memcpy(new_buff, data, len);
     free(data);
     RETURN_STRINGL(new_buff, (int) len, 0);
+#endif /* HAVE_PNG */
 }
 
 PHP_METHOD(swfed, replacePNGData) {
+#ifndef HAVE_PNG
+    fprintf(stderr, "replacePNGData: no png library\n");
+    RETURN_FALSE;
+#else  /* HAVE_PNG */
     char *data = NULL;
     int data_len = 0;
     int image_id;
@@ -585,9 +659,14 @@ PHP_METHOD(swfed, replacePNGData) {
         RETURN_FALSE;
     }
     RETURN_TRUE;
+#endif /* HAVE_PNG */
 }
 
 PHP_METHOD(swfed, replaceGIFData) {
+#ifndef HAVE_GIF
+    fprintf(stderr, "replaceGIFData: no gif library\n");
+    RETURN_FALSE;
+#else /* HAVE_GIF */
     char *data = NULL;
     int data_len = 0;
     int image_id;
@@ -611,6 +690,7 @@ PHP_METHOD(swfed, replaceGIFData) {
         RETURN_FALSE;
     }
     RETURN_TRUE;
+#endif /* HAVE_GIF */
 }
 
 PHP_METHOD(swfed, getSoundData) {
@@ -760,7 +840,6 @@ PHP_METHOD(swfed, disasmActionData) {
     }
     swf_action_list_destroy(action_list);
 }
-    
 
 PHP_METHOD(swfed, swfInfo) {
     swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
@@ -782,12 +861,12 @@ static swf_object_t  *get_swf_object(zval *obj TSRMLS_DC) {
         return NULL;
     }
     id = Z_LVAL_PP(tmp);
-    swf = zend_list_find(id, &type);
+    swf = (swf_object_t *) zend_list_find(id, &type);
     return swf;
 }
 
 static void free_swfed_resource(zend_rsrc_list_entry *resource TSRMLS_DC)
 {
 //    printf("SWFEditor->destory\n");
-    swf_object_close(resource->ptr);
+    swf_object_close((swf_object_t *) resource->ptr);
 }
